@@ -40,6 +40,8 @@
 
 #include <libedata-book/libedata-book.h>
 
+#include "lzx/ews-oal-decompress.h"
+
 #include "server/e-ews-item-change.h"
 #include "server/e-ews-message.h"
 #include "server/e-ews-connection.h"
@@ -50,8 +52,6 @@
 
 #include "e-book-backend-ews.h"
 #include "ews-oab-decoder.h"
-#include "ews-oab-decompress.h"
-
 
 #define d(x) x
 
@@ -664,7 +664,7 @@ convert_indexed_contact_property_to_updatexml (ESoapMessage *message,
 {
 	gboolean delete_field = FALSE;
 
-	if (!value || g_strcmp0 (value, "") == 0)
+	if (!value || !g_strcmp0 (value, ""))
 		delete_field = TRUE;
 	e_ews_message_start_set_indexed_item_field (message, name , prefix, "Contact", key, delete_field);
 
@@ -686,22 +686,14 @@ ebews_set_full_name_changes (ESoapMessage *message,
 
 	name = e_contact_get (new, E_CONTACT_NAME);
 	old_name = e_contact_get (old, E_CONTACT_NAME);
-	if (!old_name && !name)
+	if (!name && !old_name)
 		return;
 
-	if (!old_name) {
+	if (g_ascii_strcasecmp (name->given, old_name->given))
 		convert_contact_property_to_updatexml (message, "GivenName", name->given, "contacts", NULL, NULL);
-		convert_contact_property_to_updatexml (message, "MiddleName", name->additional, "contacts", NULL, NULL);
-	} else if (!name) {
-		convert_contact_property_to_updatexml (message, "GivenName", "", "contacts", NULL, NULL);
 
-		convert_contact_property_to_updatexml (message, "MiddleName", "", "contacts", NULL, NULL);
-	} else {
-		if (g_strcmp0 (name->given, old_name->given) != 0)
-			convert_contact_property_to_updatexml (message, "GivenName", name->given, "contacts", NULL, NULL);
-		if (g_strcmp0 (name->additional, old_name->additional) != 0)
-			convert_contact_property_to_updatexml (message, "MiddleName", name->additional, "contacts", NULL, NULL);
-	}
+	if (g_ascii_strcasecmp (name->additional, old_name->additional))
+		convert_contact_property_to_updatexml (message, "MiddleName", name->additional, "contacts", NULL, NULL);
 
 	e_contact_name_free (name);
 	e_contact_name_free (old_name);
@@ -749,12 +741,13 @@ ebews_set_phone_number_changes (ESoapMessage *message,
 	for (i = 0; i < G_N_ELEMENTS (phone_field_map); i++) {
 		new_value = e_contact_get (new, phone_field_map[i].field);
 		old_value = e_contact_get (old, phone_field_map[i].field);
-
-		if (g_strcmp0 (new_value, old_value) != 0)
+		if ((new_value && !old_value) || (!new_value && old_value) ||(new_value && old_value && g_ascii_strcasecmp (new_value, old_value)))
 			convert_indexed_contact_property_to_updatexml (message, "PhoneNumber", new_value, "contacts", "PhoneNumbers", phone_field_map[i].element);
+		if (new_value)
+			g_free (new_value);
 
-		g_free (new_value);
-		g_free (old_value);
+		if (old_value)
+			g_free (old_value);
 	}
 }
 
@@ -770,7 +763,7 @@ convert_indexed_contact_property_to_updatexml_physical_address (ESoapMessage *me
 	gchar * fielduri = NULL;
 	gboolean delete_field = FALSE;
 
-	if (!value || g_strcmp0 (value, "") == 0)
+	if (!value || !g_strcmp0 (value, ""))
 		delete_field = TRUE;
 
 	fielduri = g_strconcat (name, ":", uri_element, NULL);
@@ -816,13 +809,13 @@ compare_address (ESoapMessage *message,
 		new_address = g_new0 (EContactAddress, 1);
 	}
 
-	if (set || g_strcmp0 (new_address->street, old_address->street) != 0)
+	if (set || g_ascii_strcasecmp (new_address->street, old_address->street))
 		convert_indexed_contact_property_to_updatexml_physical_address (message, "PhysicalAddress", "Street", new_address->street, "contacts", "PhysicalAddresses", key);
-	if (set || g_strcmp0 (new_address->locality, old_address->locality) != 0)
+	if (set || g_ascii_strcasecmp (new_address->locality, old_address->locality))
 		convert_indexed_contact_property_to_updatexml_physical_address (message, "PhysicalAddress", "City", new_address->locality, "contacts", "PhysicalAddresses", key);
-	if (set || g_strcmp0 (new_address->region, old_address->region) != 0)
+	if (set || g_ascii_strcasecmp (new_address->region, old_address->region))
 		convert_indexed_contact_property_to_updatexml_physical_address (message, "PhysicalAddress", "State", new_address->region, "contacts", "PhysicalAddresses", key);
-	if (set || g_strcmp0 (new_address->code, old_address->code) != 0)
+	if (set || g_ascii_strcasecmp (new_address->code, old_address->code))
 		convert_indexed_contact_property_to_updatexml_physical_address (message, "PhysicalAddress", "PostalCode", new_address->code, "contacts", "PhysicalAddresses", key);
 
 	e_contact_address_free (old_address);
@@ -857,9 +850,9 @@ ebews_set_notes_changes (ESoapMessage *message,
 	old_notes = e_contact_get (old, E_CONTACT_NOTE);
 	new_notes = e_contact_get (new, E_CONTACT_NOTE);
 
-	if (g_strcmp0 (old_notes, new_notes) != 0) {
+	if (g_strcmp0 (old_notes, new_notes)) {
 		convert_contact_property_to_updatexml (
-				message, "Body", new_notes ? new_notes : "", "item", "BodyType", "Text");
+				message, "Body", new_notes ?: "", "item", "BodyType", "Text");
 	}
 
 	g_free (old_notes);
@@ -875,24 +868,30 @@ ebews_set_email_changes (ESoapMessage *message,
 
 	new_value = e_contact_get (new, E_CONTACT_EMAIL_1);
 	old_value = e_contact_get (old, E_CONTACT_EMAIL_1);
-	if (g_strcmp0 (new_value, old_value) != 0)
+	if ((new_value && !old_value) || (!new_value && old_value) ||(new_value && old_value && g_ascii_strcasecmp (new_value, old_value)))
 		convert_indexed_contact_property_to_updatexml (message, "EmailAddress", new_value, "contacts", "EmailAddresses", "EmailAddress1");
-	g_free (new_value);
-	g_free (old_value);
+	if (new_value)
+		g_free (new_value);
+	if (old_value)
+		g_free (old_value);
 
 	new_value = e_contact_get (new, E_CONTACT_EMAIL_2);
 	old_value = e_contact_get (old, E_CONTACT_EMAIL_2);
-	if (g_strcmp0 (new_value, old_value) != 0)
+	if ((new_value && !old_value) || (!new_value && old_value) ||(new_value && old_value && g_ascii_strcasecmp (new_value, old_value)))
 		convert_indexed_contact_property_to_updatexml (message, "EmailAddress", new_value, "contacts", "EmailAddresses", "EmailAddress2");
-	g_free (new_value);
-	g_free (old_value);
+	if (new_value)
+		g_free (new_value);
+	if (old_value)
+		g_free (old_value);
 
 	new_value = e_contact_get (new, E_CONTACT_EMAIL_3);
 	old_value = e_contact_get (old, E_CONTACT_EMAIL_3);
-	if (g_strcmp0 (new_value, old_value) != 0)
+	if ((new_value && !old_value) || (!new_value && old_value) ||(new_value && old_value && g_ascii_strcasecmp (new_value, old_value)))
 		convert_indexed_contact_property_to_updatexml (message, "EmailAddress", new_value, "contacts", "EmailAddresses", "EmailAddress3");
-	g_free (new_value);
-	g_free (old_value);
+	if (new_value)
+		g_free (new_value);
+	if (old_value)
+		g_free (old_value);
 }
 
 static const struct field_element_mapping {
@@ -926,7 +925,7 @@ static const struct field_element_mapping {
 	{ E_CONTACT_IM_AIM, ELEMENT_TYPE_COMPLEX, "ImAddresses", NULL, ebews_populate_ims, ebews_set_ims, ebews_set_im_changes },
 	{ E_CONTACT_TITLE, ELEMENT_TYPE_SIMPLE, "JobTitle", e_ews_item_get_job_title},
 	{ E_CONTACT_MANAGER, ELEMENT_TYPE_SIMPLE, "Manager", e_ews_item_get_manager},
-	{ E_CONTACT_OFFICE, ELEMENT_TYPE_SIMPLE, "OfficeLocation", e_ews_item_get_office_location},
+	{ E_CONTACT_MANAGER, ELEMENT_TYPE_SIMPLE, "OfficeLocation", e_ews_item_get_office_location},
 	{ E_CONTACT_SPOUSE, ELEMENT_TYPE_SIMPLE, "Profession", e_ews_item_get_profession},
 	{ E_CONTACT_SPOUSE, ELEMENT_TYPE_SIMPLE, "SpouseName", e_ews_item_get_spouse_name},
 	{ E_CONTACT_FAMILY_NAME, ELEMENT_TYPE_SIMPLE, "Surname", e_ews_item_get_surname},
@@ -1183,7 +1182,7 @@ e_book_backend_ews_remove_contacts (EBookBackend *backend,
 		return;
 	}
 
-	for (l = (GSList *) id_list; l != NULL; l = g_slist_next (l))
+	for (l = (GSList *) id_list; l != NULL; l = g_slist_next (id_list))
 		copy = g_slist_prepend (copy, g_strdup ((gchar *) l->data));
 	copy = g_slist_reverse (copy);
 
@@ -1296,7 +1295,7 @@ convert_contact_to_updatexml (ESoapMessage *msg,
 		if (element_type == ELEMENT_TYPE_SIMPLE)  {
 			value =  e_contact_get (new_contact, mappings[i].field_id);
 			old_value =  e_contact_get (old_contact, mappings[i].field_id);
-			if (g_strcmp0 (value, old_value) != 0)
+			if ((value && !old_value) || (!value && old_value) ||(value && old_value && g_ascii_strcasecmp (value, old_value)))
 				convert_contact_property_to_updatexml (msg, mappings[i].element_name, value, "contacts", NULL, NULL);
 			if (value)
 				g_free (value);
@@ -1727,50 +1726,31 @@ e_book_backend_ews_build_restriction (const gchar *query,
 
 /************* GAL sync ***********************/
 
-static gchar *
-ews_download_gal_file (EBookBackendEws *cbews,
-                       EwsOALDetails *full,
-                       GCancellable *cancellable,
-                       GError **error)
+static gboolean
+ews_gal_needs_update (EBookBackendEws *cbews,
+                      EwsOALDetails *full,
+                      GError **error)
 {
 	EBookBackendEwsPrivate *priv = cbews->priv;
-	EEwsConnection *oab_cnc;
-	gchar *full_url, *oab_url;
-	const gchar *cache_dir;
-	gchar *download_path = NULL;
-	gchar *password;
-	CamelEwsSettings *ews_settings;
+	guint32 seq;
+	gboolean ret = FALSE;
+	gchar *tmp;
 
-	ews_settings = book_backend_ews_get_collection_settings (cbews);
+	if (!priv->summary)
+		return FALSE;
 
-	/* oab url with oab.xml removed from the suffix */
-	oab_url = g_strndup (priv->oab_url, strlen (priv->oab_url) - 7);
-	full_url = g_strconcat (oab_url, full->filename, NULL);
-	cache_dir = e_book_backend_get_cache_dir (E_BOOK_BACKEND (cbews));
-	download_path = g_build_filename (cache_dir, full->filename, NULL);
-
-	oab_cnc = e_ews_connection_new (full_url, ews_settings);
-
-	password = e_ews_connection_dup_password (priv->cnc);
-	e_ews_connection_set_password (oab_cnc, password);
-	g_free (password);
-
-	if (!e_ews_connection_download_oal_file_sync (oab_cnc, download_path,
-						      NULL, NULL,
-						      cancellable, error)) {
-		g_free (download_path);
-		download_path = NULL;
+	tmp = e_book_backend_sqlitedb_get_key_value (priv->summary, priv->folder_id, "seq", error);
+	if (error)
 		goto exit;
-	}
 
-	d (g_print ("OAL file downloaded %s\n", download_path);)
+	sscanf (tmp, "%"G_GUINT32_FORMAT, &seq);
+	if (seq < full->seq)
+		ret = TRUE;
 
- exit:
-	g_object_unref (oab_cnc);
-	g_free (oab_url);
-	g_free (full_url);
-
-	return download_path;
+	d (printf ("Gal needs update: %d \n", ret);)
+exit:
+	g_free (tmp);
+	return ret;
 }
 
 static gchar *
@@ -1780,102 +1760,51 @@ ews_download_full_gal (EBookBackendEws *cbews,
                        GError **error)
 {
 	EBookBackendEwsPrivate *priv = cbews->priv;
+	EEwsConnection *oab_cnc;
+	gchar *full_url, *oab_url, *cache_file = NULL;
 	const gchar *cache_dir;
-	gchar *lzx_path, *oab_file, *oab_path;
+	gchar *comp_cache_file = NULL, *uncompress_file = NULL;
+	gchar *password;
+	CamelEwsSettings *ews_settings;
 
-	lzx_path = ews_download_gal_file (cbews, full, cancellable, error);
-	if (!lzx_path)
-		return NULL;
+	ews_settings = book_backend_ews_get_collection_settings (cbews);
 
+	/* oab url with oab.xml removed from the suffix */
+	oab_url = g_strndup (priv->oab_url, strlen (priv->oab_url) - 7);
+	full_url = g_strconcat (oab_url, full->filename, NULL);
 	cache_dir = e_book_backend_get_cache_dir (E_BOOK_BACKEND (cbews));
-	oab_file = g_strdup_printf ("%s-%d.oab", priv->folder_name, full->seq);
-	oab_path = g_build_filename (cache_dir, oab_file, NULL);
-	if (!ews_oab_decompress_full (lzx_path, oab_path, error)) {
-		g_free (oab_path);
-		oab_path = NULL;
+	comp_cache_file = g_build_filename (cache_dir, full->filename, NULL);
+
+	oab_cnc = e_ews_connection_new (full_url, ews_settings);
+
+	password = e_ews_connection_dup_password (priv->cnc);
+	e_ews_connection_set_password (oab_cnc, password);
+	g_free (password);
+
+	if (!e_ews_connection_download_oal_file_sync (
+		oab_cnc, comp_cache_file, NULL, NULL, cancellable, error))
+		goto exit;
+
+	cache_file = g_strdup_printf ("%s-%d.oab", priv->folder_name, full->seq);
+	uncompress_file = g_build_filename (cache_dir, cache_file, NULL);
+	if (!oal_decompress_v4_full_detail_file (comp_cache_file, uncompress_file, error)) {
+		g_free (uncompress_file);
+		uncompress_file = NULL;
 		goto exit;
 	}
 
-	d (g_print ("OAL file decompressed %s \n", oab_path);)
+	d (g_print ("OAL file decompressed %s \n", uncompress_file);)
 
 exit:
-	if (lzx_path) {
-		g_unlink (lzx_path);
-		g_free (lzx_path);
-	}
-	g_free (oab_file);
-	return oab_path;
-}
+	if (comp_cache_file)
+		g_unlink (comp_cache_file);
+	g_object_unref (oab_cnc);
+	g_free (oab_url);
+	g_free (full_url);
+	g_free (comp_cache_file);
+	g_free (cache_file);
 
-static gchar *
-ews_download_gal (EBookBackendEws *cbews, EwsOALDetails *full, GSList *deltas, guint32 seq,
-		  GCancellable *cancellable, GError **error)
-{
-#ifdef USE_MSPACK
-	EBookBackendEwsPrivate *priv = cbews->priv;
-	GSList *p;
-	gchar *thisoab = NULL;
-	const gchar *cache_dir;
-
-	cache_dir = e_book_backend_get_cache_dir (E_BOOK_BACKEND (cbews));
-
-	thisoab = e_book_backend_sqlitedb_get_key_value (priv->summary, priv->folder_id,
-							 "oab-filename", NULL);
-	if (!thisoab)
-		goto full;
-
-	for (p = deltas; p; p = p->next) {
-		EwsOALDetails *det = p->data;
-		GError *local_error = NULL;
-		gchar *oab_file, *lzx_path, *nextoab;
-
-		seq++;
-		if (det->seq != seq)
-			break;
-
-		lzx_path = ews_download_gal_file (cbews, det, cancellable, NULL);
-		if (!lzx_path)
-			break;
-
-		oab_file = g_strdup_printf ("%s-%d.oab", priv->folder_name, seq);
-		nextoab = g_build_filename (cache_dir, oab_file, NULL);
-		g_free (oab_file);
-
-		ews_oab_decompress_patch (lzx_path, thisoab, nextoab, &local_error);
-
-		/* Free the LZX file */
-		g_unlink (lzx_path);
-		g_free (lzx_path);
-
-		/* Free the previous OAB file */
-		g_unlink (thisoab);
-		g_free (thisoab);
-
-		thisoab = nextoab;
-
-		/* For once we are *allowed* to use the error instead of having to
-		 * check the return value of the function. It's our *own* error. */
-		if (local_error) {
-			d (g_print ("Failed to apply incremental patch: %s\n",
-				    local_error->message));
-			g_error_free (local_error);
-			break;
-		}
-
-		d (g_print ("Created %s from delta\n", thisoab));
-
-		if (seq == full->seq)
-			return thisoab;
-	}
-
-	if (thisoab) {
-		g_unlink (thisoab);
-		g_free (thisoab);
-	}
- full:
-#endif
-	d (printf ("Ewsgal: Downloading full gal \n"));
-	return ews_download_full_gal (cbews, full, cancellable, error);
+	return uncompress_file;
 }
 
 static gboolean
@@ -1950,13 +1879,6 @@ ews_gal_store_contact (EContact *contact,
 		e_book_backend_notify_complete (E_BOOK_BACKEND (data->cbews));
 }
 
-static gint det_sort_func (gconstpointer _a, gconstpointer _b)
-{
-	const EwsOALDetails *a = _a, *b = _b;
-
-	return a->seq - b->seq;
-}
-
 static gboolean
 ews_replace_gal_in_db (EBookBackendEws *cbews,
                        const gchar *filename,
@@ -2013,15 +1935,9 @@ ebews_start_gal_sync (gpointer data)
 	GError *error = NULL;
 	EEwsConnection *oab_cnc;
 	GSList *full_l = NULL;
-	GSList *deltas = NULL;
 	gboolean ret = TRUE;
-	gboolean is_populated;
 	gchar *uncompressed_filename = NULL;
 	gchar *password;
-	gchar *old_etag = NULL, *etag = NULL;
-	gchar *seq;
-	guint32 old_seq = 0;
-	guint32 delta_size = 0;
 	CamelEwsSettings *ews_settings;
 
 	cbews = (EBookBackendEws *) data;
@@ -2036,22 +1952,8 @@ ebews_start_gal_sync (gpointer data)
 
 	d (printf ("Ewsgal: Fetching oal full details file \n");)
 
-	is_populated = e_book_backend_sqlitedb_get_is_populated (priv->summary, priv->folder_id, NULL);
-	if (is_populated) {
-		gchar *tmp;
-		old_etag = e_book_backend_sqlitedb_get_key_value (
-			priv->summary, priv->folder_id, "etag", NULL);
-		tmp = e_book_backend_sqlitedb_get_key_value (
-			priv->summary, priv->folder_id, "seq", NULL);
-		if (tmp)
-			old_seq = strtoul(tmp, NULL, 10);
-		else
-			is_populated = FALSE;
-		g_free (tmp);
-	}
-
 	if (!e_ews_connection_get_oal_detail_sync (
-		oab_cnc, priv->folder_id, NULL, old_etag, &full_l, &etag,
+		oab_cnc, priv->folder_id, "Full", &full_l,
 		priv->cancellable, &error)) {
 		ret = FALSE;
 		goto exit;
@@ -2064,71 +1966,41 @@ ebews_start_gal_sync (gpointer data)
 	if (full_l == NULL)
 		goto exit;
 
-	while (full_l) {
-		EwsOALDetails *det = full_l->data;
+	full = (EwsOALDetails *) full_l->data;
+	/* TODO fetch differential updates if available instead of downloading the whole GAL */
+	if (!e_book_backend_sqlitedb_get_is_populated (priv->summary, priv->folder_id, NULL) || ews_gal_needs_update (cbews, full, &error)) {
+		gchar *seq;
 
-		/* Throw away anything older than we already have */
-		if (det->seq <= old_seq) {
-			ews_oal_details_free (det);
-		} else if (!strcmp (det->type, "Full")) {
-			if (full)
-				ews_oal_details_free (full);
-			full = det;
-		} else if (is_populated && !strcmp (det->type, "Diff")) {
-			delta_size += det->size;
-			deltas = g_slist_insert_sorted (deltas, det, det_sort_func);
-		} else {
-			ews_oal_details_free (det);
+		d (printf ("Ewsgal: Downloading full gal \n");)
+		uncompressed_filename = ews_download_full_gal (cbews, full, priv->cancellable, &error);
+		if (error) {
+			ret = FALSE;
+			goto exit;
+		}
+
+		d (printf ("Ewsgal: Removing old gal \n");)
+		/* remove old_gal_file */
+		ret = ews_remove_old_gal_file (cbews, &error);
+		if (!ret) {
+			goto exit;
+		}
+
+		d (printf ("Ewsgal: Replacing old gal with new gal contents in db \n");)
+		ret = ews_replace_gal_in_db (cbews, uncompressed_filename, priv->cancellable, &error);
+		if (!ret)
+			goto exit;
+
+		seq = g_strdup_printf ("%"G_GUINT32_FORMAT, full->seq);
+		ret = e_book_backend_sqlitedb_set_key_value (priv->summary, priv->folder_id, "seq", seq, &error);
+		g_free (seq);
+
+		if (!ret) {
+			e_book_backend_sqlitedb_delete_addressbook (priv->summary, priv->folder_id, &error);
+			goto exit;
 		}
-		full_l = g_slist_remove (full_l, det);
 	}
 
-	if (!full)
-		goto exit;
-
-	/* If the deltas would be bigger, just download the new full file */
-	if (delta_size > full->size) {
-		g_slist_free_full (deltas, (GDestroyNotify) ews_oal_details_free);
-		deltas = NULL;
-	}
-
-	uncompressed_filename = ews_download_gal (cbews, full, deltas, old_seq, priv->cancellable, &error);
-	if (!uncompressed_filename) {
-		ret = FALSE;
-		goto exit;
-	}
-
-	d (printf ("Ewsgal: Removing old gal \n");)
-	/* remove old_gal_file */
-	ret = ews_remove_old_gal_file (cbews, &error);
-	if (!ret) {
-		goto exit;
-	}
-
-	d (printf ("Ewsgal: Replacing old gal with new gal contents in db \n");)
-	ret = ews_replace_gal_in_db (cbews, uncompressed_filename, priv->cancellable, &error);
-	if (!ret)
-		goto exit;
-
-	e_book_backend_sqlitedb_set_key_value (priv->summary, priv->folder_id, "etag", etag ? etag : "", NULL);
-	if (e_book_backend_sqlitedb_set_key_value (priv->summary, priv->folder_id,
-						   "oab-filename", uncompressed_filename,
-						   NULL)) {
-		/* Don't let it get deleted */
-		g_free (uncompressed_filename);
-		uncompressed_filename = NULL;
-	}
-
-	seq = g_strdup_printf ("%"G_GUINT32_FORMAT, full->seq);
-	ret = e_book_backend_sqlitedb_set_key_value (priv->summary, priv->folder_id, "seq", seq, &error);
-	g_free (seq);
-
-	if (!ret) {
-		e_book_backend_sqlitedb_delete_addressbook (priv->summary, priv->folder_id, &error);
-		goto exit;
-	}
-
-	d (printf ("Ews gal: sync successful complete \n");)
+	d (printf ("Ews gal: sync successfull complete \n");)
 
 exit:
 	if (error) {
@@ -2136,21 +2008,18 @@ exit:
 		g_clear_error (&error);
 	}
 
-	g_free (old_etag);
-	g_free (etag);
-
 	/* preserve  the oab file once we are able to decode the differential updates */
 	if (uncompressed_filename) {
 		g_unlink (uncompressed_filename);
 		g_free (uncompressed_filename);
 	}
 
-	if (full)
-		ews_oal_details_free (full);
-	if (deltas)
-		g_slist_free_full (deltas, (GDestroyNotify) ews_oal_details_free);
-	if (full_l)
-		g_slist_free_full (full_l, (GDestroyNotify) ews_oal_details_free);
+	if (full_l) {
+		g_free (full->sha);
+		g_free (full->filename);
+		g_free (full);
+		g_slist_free (full_l);
+	}
 
 	g_object_unref (oab_cnc);
 	return ret;
@@ -2723,7 +2592,9 @@ fetch_from_offline (EBookBackendEws *ews,
 	for (l = contacts; l != NULL; l = g_slist_next (l)) {
 		EbSdbSearchData *s_data = (EbSdbSearchData *) l->data;
 
+		/* reset vcard to NULL as it would be free'ed in prefiltered_vcard function */
 		e_data_book_view_notify_update_prefiltered_vcard (book_view, s_data->uid, s_data->vcard);
+		s_data->vcard = NULL;
 
 		e_book_backend_sqlitedb_search_data_free (s_data);
 	}
@@ -2745,7 +2616,7 @@ e_book_backend_ews_start_view (EBookBackend *backend,
 	gboolean is_autocompletion = FALSE;
 	gchar *auto_comp_str = NULL;
 	GCancellable *cancellable;
-	GSList *mailboxes = NULL, *l, *contacts = NULL, *c;
+	GSList *ids = NULL, *mailboxes = NULL, *l, *contacts = NULL, *c;
 	EwsFolderId *fid;
 	ESource *source;
 	ESourceEwsFolder *extension;
@@ -2821,6 +2692,7 @@ e_book_backend_ews_start_view (EBookBackend *backend,
 	fid = g_new0 (EwsFolderId, 1);
 	fid->id = g_strdup (priv->folder_id);
 	fid->change_key = e_source_ews_folder_dup_change_key (extension);
+	ids = g_slist_append (ids, fid);
 
 	/* We do not scan until we reach the last_item as it might be good enough to show first 100
 	 * items during auto-completion. Change it if needed. TODO, Personal Address-book should start using
@@ -3000,66 +2872,75 @@ e_book_backend_ews_notify_online_cb (EBookBackend *backend,
 	}
 }
 
-static void
+static gchar *
 e_book_backend_ews_get_backend_property (EBookBackend *backend,
-                                         EDataBook *book,
-                                         guint32 opid,
-                                         GCancellable *cancellable,
                                          const gchar *prop_name)
 {
-	g_return_if_fail (prop_name != NULL);
+	g_return_val_if_fail (prop_name != NULL, NULL);
 
 	if (g_str_equal (prop_name, CLIENT_BACKEND_PROPERTY_CAPABILITIES)) {
 		EBookBackendEws *ebews;
 
 		ebews = E_BOOK_BACKEND_EWS (backend);
-		g_return_if_fail (ebews != NULL);
+		g_return_val_if_fail (ebews != NULL, NULL);
 
 		if (ebews->priv->is_gal) {
-			e_data_book_respond_get_backend_property (book, opid, NULL, "net,bulk-removes,contact-lists");
+			return g_strdup ("net,bulk-removes,contact-lists");
 		} else {
 			/* do-initialy-query is enabled for system address book also, so that we get the
 			 * book_view, which is needed for displaying cache update progress.
 			 * and null query is handled for system address book.
 			 */
-			e_data_book_respond_get_backend_property (book, opid, NULL, "net,bulk-removes,do-initial-query,contact-lists");
+			return g_strdup ("net,bulk-removes,do-initial-query,contact-lists");
 		}
 	} else if (g_str_equal (prop_name, BOOK_BACKEND_PROPERTY_REQUIRED_FIELDS)) {
-		e_data_book_respond_get_backend_property (book, opid, NULL, e_contact_field_name (E_CONTACT_FILE_AS));
+		return g_strdup (e_contact_field_name (E_CONTACT_FILE_AS));
 	} else if (g_str_equal (prop_name, BOOK_BACKEND_PROPERTY_SUPPORTED_FIELDS)) {
-		gchar *fields_str;
-		GSList *fields = NULL;
-		gint i;
+		GString *buffer;
+		gchar *fields;
+		gint ii;
 
-		for (i = 0; i < G_N_ELEMENTS (mappings); i++)
-			if (mappings[i].element_type == ELEMENT_TYPE_SIMPLE)
-				fields = g_slist_append (fields, (gpointer) e_contact_field_name (mappings[i].field_id));
+		buffer = g_string_sized_new (1024);
 
-		for (i = 0; i < G_N_ELEMENTS (phone_field_map); i++)
-			fields = g_slist_append (fields, (gpointer) e_contact_field_name (phone_field_map[i].field));
+		for (ii = 0; ii < G_N_ELEMENTS (mappings); ii++) {
+			if (mappings[ii].element_type != ELEMENT_TYPE_SIMPLE)
+				continue;
 
-		fields = g_slist_append (fields, (gpointer) e_contact_field_name (E_CONTACT_FULL_NAME));
-		fields = g_slist_append (fields, (gpointer) e_contact_field_name (E_CONTACT_NICKNAME));
-		fields = g_slist_append (fields, (gpointer) e_contact_field_name (E_CONTACT_FAMILY_NAME));
-		fields = g_slist_append (fields, (gpointer) e_contact_field_name (E_CONTACT_EMAIL_1));
-		fields = g_slist_append (fields, (gpointer) e_contact_field_name (E_CONTACT_EMAIL_2));
-		fields = g_slist_append (fields, (gpointer) e_contact_field_name (E_CONTACT_EMAIL_3));
-		fields = g_slist_append (fields, (gpointer) e_contact_field_name (E_CONTACT_ADDRESS_WORK));
-		fields = g_slist_append (fields, (gpointer) e_contact_field_name (E_CONTACT_ADDRESS_HOME));
-		fields = g_slist_append (fields, (gpointer) e_contact_field_name (E_CONTACT_ADDRESS_OTHER));
-		fields = g_slist_append (fields, (gpointer) e_contact_field_name (E_CONTACT_BIRTH_DATE));
+			if (buffer->len > 0)
+				g_string_append_c (buffer, ',');
+			g_string_append (buffer, e_contact_field_name (mappings[ii].field_id));
+		}
 
-		fields = g_slist_append (fields, (gpointer) e_contact_field_name (E_CONTACT_NOTE));
+		for (ii = 0; ii < G_N_ELEMENTS (phone_field_map); ii++) {
+			if (buffer->len > 0)
+				g_string_append_c (buffer, ',');
+			g_string_append (buffer, e_contact_field_name (phone_field_map[ii].field));
+		}
 
-		fields_str = e_data_book_string_slist_to_comma_string (fields);
+		fields = g_strjoin (
+			",",
+			buffer->str,
+			e_contact_field_name (E_CONTACT_FULL_NAME),
+			e_contact_field_name (E_CONTACT_NICKNAME),
+			e_contact_field_name (E_CONTACT_FAMILY_NAME),
+			e_contact_field_name (E_CONTACT_EMAIL_1),
+			e_contact_field_name (E_CONTACT_EMAIL_2),
+			e_contact_field_name (E_CONTACT_EMAIL_3),
+			e_contact_field_name (E_CONTACT_ADDRESS_WORK),
+			e_contact_field_name (E_CONTACT_ADDRESS_HOME),
+			e_contact_field_name (E_CONTACT_ADDRESS_OTHER),
+			e_contact_field_name (E_CONTACT_BIRTH_DATE),
+			e_contact_field_name (E_CONTACT_NOTE),
+			NULL);
 
-		e_data_book_respond_get_backend_property (book, opid, NULL, fields_str);
+		g_string_free (buffer, TRUE);
 
-		g_slist_free (fields);
-		g_free (fields_str);
-	} else {
-		E_BOOK_BACKEND_CLASS (e_book_backend_ews_parent_class)->get_backend_property (backend, book, opid, cancellable, prop_name);
+		return fields;
 	}
+
+	/* Chain up to parent's get_backend_property() method. */
+	return E_BOOK_BACKEND_CLASS (e_book_backend_ews_parent_class)->
+		get_backend_property (backend, prop_name);
 }
 
 static void
@@ -3172,26 +3053,8 @@ e_book_backend_ews_dispose (GObject *object)
 	bews = E_BOOK_BACKEND_EWS (object);
 	priv = bews->priv;
 
-	if (priv->cancellable)
-		g_cancellable_cancel (priv->cancellable);
-
-	if (priv->dlock) {
-		g_mutex_lock (&priv->dlock->mutex);
-		priv->dlock->exit = TRUE;
-		g_mutex_unlock (&priv->dlock->mutex);
-
-		g_cond_signal (&priv->dlock->cond);
-
-		if (priv->dthread)
-			g_thread_join (priv->dthread);
-
-		g_mutex_clear (&priv->dlock->mutex);
-		g_cond_clear (&priv->dlock->cond);
-		g_free (priv->dlock);
-		priv->dthread = NULL;
-	}
-
 	if (priv->cancellable) {
+		g_cancellable_cancel (priv->cancellable);
 		g_object_unref (priv->cancellable);
 		priv->cancellable = NULL;
 	}
@@ -3221,6 +3084,22 @@ e_book_backend_ews_dispose (GObject *object)
 		priv->attachment_dir = NULL;
 	}
 
+	if (priv->dlock) {
+		g_mutex_lock (&priv->dlock->mutex);
+		priv->dlock->exit = TRUE;
+		g_mutex_unlock (&priv->dlock->mutex);
+
+		g_cond_signal (&priv->dlock->cond);
+
+		if (priv->dthread)
+			g_thread_join (priv->dthread);
+
+		g_mutex_clear (&priv->dlock->mutex);
+		g_cond_clear (&priv->dlock->cond);
+		g_free (priv->dlock);
+		priv->dthread = NULL;
+	}
+
 	if (priv->summary) {
 		g_object_unref (priv->summary);
 		priv->summary = NULL;
