@@ -44,11 +44,13 @@
 #include "server/camel-ews-settings.h"
 #include "server/e-ews-item-change.h"
 #include "server/e-ews-message.h"
+#include "server/e-ews-oof-settings.h"
 
 #include "camel-ews-folder.h"
 #include "camel-ews-store.h"
 #include "camel-ews-summary.h"
 #include "camel-ews-utils.h"
+#include "camel-ews-enumtypes.h"
 
 #ifdef G_OS_WIN32
 #include <winsock2.h>
@@ -69,6 +71,8 @@ struct _CamelEwsStorePrivate {
 	GMutex get_finfo_lock;
 	EEwsConnection *connection;
 	GMutex connection_lock;
+	gboolean has_ooo_set;
+	CamelEwsStoreOooAlertState ooo_alert_state;
 	GSList *public_folders; /* EEwsFolder * objects */
 };
 
@@ -79,6 +83,12 @@ static void camel_ews_store_initable_init (GInitableIface *interface);
 static void camel_ews_subscribable_init (CamelSubscribableInterface *interface);
 static GInitableIface *parent_initable_interface;
 
+enum {
+	PROP_0,
+	PROP_HAS_OOO_SET,
+	PROP_OOO_ALERT_STATE
+};
+
 G_DEFINE_TYPE_WITH_CODE (
 	CamelEwsStore, camel_ews_store, CAMEL_TYPE_OFFLINE_STORE,
 	G_IMPLEMENT_INTERFACE (
@@ -86,6 +96,96 @@ G_DEFINE_TYPE_WITH_CODE (
 	G_IMPLEMENT_INTERFACE (
 		CAMEL_TYPE_SUBSCRIBABLE, camel_ews_subscribable_init))
 
+static void
+ews_store_set_property (GObject *object,
+                        guint property_id,
+                        const GValue *value,
+                        GParamSpec *pspec)
+{
+	switch (property_id) {
+		case PROP_HAS_OOO_SET:
+			camel_ews_store_set_has_ooo_set (
+				CAMEL_EWS_STORE (object),
+				g_value_get_boolean (value));
+			return;
+		case PROP_OOO_ALERT_STATE:
+			camel_ews_store_set_ooo_alert_state (
+				CAMEL_EWS_STORE (object),
+				g_value_get_enum (value));
+			return;
+	}
+
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+}
+
+static void
+ews_store_get_property (GObject *object,
+                        guint property_id,
+                        GValue *value,
+                        GParamSpec *pspec)
+{
+	switch (property_id) {
+		case PROP_HAS_OOO_SET:
+			g_value_set_boolean (
+				value,
+				camel_ews_store_get_has_ooo_set (
+				CAMEL_EWS_STORE (object)));
+			return;
+		case PROP_OOO_ALERT_STATE:
+			g_value_set_enum (
+				value,
+				camel_ews_store_get_ooo_alert_state (
+				CAMEL_EWS_STORE (object)));
+			return;
+	}
+
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+}
+
+void
+camel_ews_store_set_has_ooo_set (CamelEwsStore *ews_store,
+                                 gboolean has_ooo_set)
+{
+	g_return_if_fail (CAMEL_IS_EWS_STORE (ews_store));
+
+	if ((ews_store->priv->has_ooo_set ? 1 : 0) == (has_ooo_set ? 1 : 0))
+		return;
+
+	ews_store->priv->has_ooo_set = has_ooo_set;
+	g_object_notify (G_OBJECT (ews_store), "has-ooo-set");
+}
+
+gboolean
+camel_ews_store_get_has_ooo_set (const CamelEwsStore *ews_store)
+{
+	g_return_val_if_fail (CAMEL_IS_EWS_STORE (ews_store), FALSE);
+
+	return ews_store->priv->has_ooo_set;
+}
+
+void
+camel_ews_store_set_ooo_alert_state (CamelEwsStore *ews_store,
+                                     CamelEwsStoreOooAlertState state)
+{
+	g_return_if_fail (CAMEL_IS_EWS_STORE (ews_store));
+
+	if (ews_store->priv->ooo_alert_state == state)
+		return;
+
+	ews_store->priv->ooo_alert_state = state;
+	g_object_notify (G_OBJECT (ews_store), "ooo-alert-state");
+}
+
+CamelEwsStoreOooAlertState
+camel_ews_store_get_ooo_alert_state (const CamelEwsStore *ews_store)
+{
+	g_return_val_if_fail (
+			CAMEL_IS_EWS_STORE (ews_store),
+			CAMEL_EWS_STORE_OOO_ALERT_STATE_UNKNOWN);
+
+	return ews_store->priv->ooo_alert_state;
+}
+
 static void
 ews_migrate_to_user_cache_dir (CamelService *service)
 {
@@ -488,6 +588,43 @@ ews_update_folder_hierarchy (CamelEwsStore *ews_store,
 	g_free (sync_state);
 }
 
+static void
+ews_update_has_ooo_set (CamelSession *session,
+			GCancellable *cancellable,
+			gpointer user_data,
+			GError **error)
+{
+
+	CamelEwsStore *ews_store = user_data;
+	EEwsOofSettings *oof_settings;
+	EEwsOofState oof_state;
+	GError *local_error = NULL;
+
+	camel_operation_push_message (cancellable, _("Checking \"Out of Office\" settings"));
+
+	oof_settings = e_ews_oof_settings_new_sync (ews_store->priv->connection, cancellable, &local_error);
+	if (local_error != NULL) {
+		g_propagate_error (error, local_error);
+		camel_operation_pop_message (cancellable);
+		return;
+	}
+
+	oof_state = e_ews_oof_settings_get_state (oof_settings);
+	switch (oof_state) {
+		case E_EWS_OOF_STATE_ENABLED:
+			camel_ews_store_set_has_ooo_set (ews_store, TRUE);
+			break;
+		case E_EWS_OOF_STATE_DISABLED:
+		case E_EWS_OOF_STATE_SCHEDULED:
+			camel_ews_store_set_has_ooo_set (ews_store, FALSE);
+			break;
+		default:
+			break;
+	}
+
+	camel_operation_pop_message (cancellable);
+}
+
 static gboolean
 ews_connect_sync (CamelService *service,
                   GCancellable *cancellable,
@@ -496,8 +633,6 @@ ews_connect_sync (CamelService *service,
 	EEwsConnection *connection;
 	CamelEwsStore *ews_store;
 	CamelSession *session;
-	CamelSettings *settings;
-	gchar *auth_mech;
 	gboolean success;
 
 	ews_store = CAMEL_EWS_STORE (service);
@@ -512,56 +647,29 @@ ews_connect_sync (CamelService *service,
 	}
 
 	session = camel_service_ref_session (service);
-	settings = camel_service_ref_settings (service);
 
 	/* Try running an operation that requires authentication
-	 * to make sure we have valid credentials available. */
-	auth_mech = camel_network_settings_dup_auth_mechanism (
-				       CAMEL_NETWORK_SETTINGS (settings));
-	if (!auth_mech)
-		auth_mech = g_strdup ("NTLM");
- retry:
-	success = camel_session_authenticate_sync (session, service,
-			   auth_mech, cancellable, error);
+	 * to make sure we have a valid password available. */
+	success = camel_session_authenticate_sync (
+		session, service, NULL, cancellable, error);
 
-	g_free (auth_mech);
+	if (success) {
+		CamelEwsStoreOooAlertState state;
 
-#if 1 /* Temporary workaround for Evolution < 3.8.4 */
-	connection = camel_ews_store_ref_connection (ews_store);
-	if (connection) {
-		g_object_unref (connection);
-	} else if (success && auth_mech) {
-		/* This should never happen, but it does with Evolution 3.8.3
-		   and older because it doesn't actually *call* our
-		   authenticate_sync() method if it thinks that Single-Sign-On
-		   *might* work with the selected authentication mechanism.
-		   Fixed with commit 447f7d38 in Evolution, but we work around
-		   it for now for better compatibility.... */
-		CamelAuthenticationResult result;
+		state = camel_ews_store_get_ooo_alert_state (ews_store);
+		if (state == CAMEL_EWS_STORE_OOO_ALERT_STATE_UNKNOWN)
+			camel_session_submit_job (
+				session,
+				ews_update_has_ooo_set,
+				g_object_ref (ews_store),
+				g_object_unref);
 
-		g_print("Trying single-sign-on directly.\n");
-		result = camel_service_authenticate_sync(service, NULL,
-							 cancellable, error);
-		/* It appeared to be available, but didn't actually work.
-		   So fall back to actually *asking* for a password... */
-		if (result != CAMEL_AUTHENTICATION_ACCEPTED) {
-			g_print("SSO not working. Falling back to password\n");
-			auth_mech = NULL;
-			goto retry;
-		}
-		g_print("SSO login succeeded\n");
-	}
-#endif
-
-	g_object_unref (settings);
-
-	g_object_unref (session);
-
-	if (success)
 		camel_offline_store_set_online_sync (
 			CAMEL_OFFLINE_STORE (ews_store),
 			TRUE, cancellable, NULL);
+	}
 
+	g_object_unref (session);
 	return success;
 }
 
@@ -1005,6 +1113,14 @@ ews_authenticate_sync (CamelService *service,
 
 	password = camel_service_get_password (service);
 
+	if (password == NULL) {
+		g_set_error_literal (
+			error, CAMEL_SERVICE_ERROR,
+			CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+			_("Authentication password not available"));
+		return CAMEL_AUTHENTICATION_ERROR;
+	}
+
 	settings = camel_service_ref_settings (service);
 
 	ews_settings = CAMEL_EWS_SETTINGS (settings);
@@ -1611,19 +1727,13 @@ ews_get_folder_info_sync (CamelStore *store,
 	}
 
 	g_mutex_lock (&priv->get_finfo_lock);
-	if (!camel_offline_store_get_online (CAMEL_OFFLINE_STORE (store))) {
+	if (!(camel_offline_store_get_online (CAMEL_OFFLINE_STORE (store))
+	      && camel_service_connect_sync ((CamelService *) store, cancellable, error))) {
 		camel_ews_store_ensure_virtual_folders (ews_store);
 		g_mutex_unlock (&priv->get_finfo_lock);
 		goto offline;
 	}
 
-	if (!camel_service_connect_sync ((CamelService *) store, cancellable, error)) {
-		camel_offline_store_set_online_sync (CAMEL_OFFLINE_STORE (store), FALSE, NULL, NULL);
-		camel_ews_store_ensure_virtual_folders (ews_store);
-		g_mutex_unlock (&priv->get_finfo_lock);
-		return NULL;
-	}
-
 	old_sync_state = camel_ews_store_summary_get_string_val (ews_store->summary, "sync_state", NULL);
 	if (!old_sync_state)
 		initial_setup = TRUE;
@@ -1654,6 +1764,8 @@ ews_get_folder_info_sync (CamelStore *store,
 		camel_ews_store_summary_store_string_val (ews_store->summary, "sync_state", "");
 		camel_ews_store_summary_clear (ews_store->summary);
 
+		initial_setup = TRUE;
+
 		success = e_ews_connection_sync_folder_hierarchy_sync (connection, EWS_PRIORITY_MEDIUM, NULL,
 			&new_sync_state, &includes_last_folder, &folders_created, &folders_updated, &folders_deleted,
 			cancellable, &local_error);
@@ -1979,7 +2091,7 @@ ews_rename_folder_sync (CamelStore *store,
 		/* Folder basename changed (i.e. UpdateFolder needed).
 		 * Therefore, we can only do it if the folder hasn't also
 		 * been moved from one parent folder to another.
-		 *
+ *
 		 * Strictly speaking, we could probably handle this, even
 		 * if there are name collisions. We could UpdateFolder to
 		 * a new temporary name that doesn't exist in either the
@@ -2135,37 +2247,12 @@ ews_get_trash_folder_sync (CamelStore *store,
 	folder_name = camel_ews_store_summary_get_folder_full_name (
 		ews_store->summary, folder_id, NULL);
 
-	folder = camel_store_get_folder_sync (
+	folder = ews_get_folder_sync (
 		store, folder_name, 0, cancellable, error);
 
 	g_free (folder_name);
 	g_free (folder_id);
 
-	if (folder) {
-		GPtrArray *folders;
-		gboolean can = TRUE;
-		gint ii;
-
-		/* Save content of all opened folders, thus any messages deleted in them
-		   are moved to the Deleted Items folder first, thus in case of the trash
-		   folder instance being used to expunge messages will contain all of them.
-		*/
-		folders = camel_object_bag_list (store->folders);
-		for (ii = 0; ii < folders->len; ii++) {
-			CamelFolder *secfolder = folders->pdata[ii];
-
-			if (secfolder != folder && can)
-			    can = camel_folder_synchronize_sync (secfolder, FALSE, cancellable, NULL);
-
-			g_object_unref (secfolder);
-		}
-		g_ptr_array_free (folders, TRUE);
-
-		/* To return 'Deleted Items' folder with current content,
-		   not with possibly stale locally cached copy. */
-		camel_folder_refresh_info_sync (folder, cancellable, NULL);
-	}
-
 	return folder;
 }
 
@@ -2195,7 +2282,7 @@ ews_get_junk_folder_sync (CamelStore *store,
 	folder_name = camel_ews_store_summary_get_folder_full_name (
 		ews_store->summary, folder_id, NULL);
 
-	folder = camel_store_get_folder_sync (
+	folder = ews_get_folder_sync (
 		store, folder_name, 0, cancellable, error);
 
 	g_free (folder_name);
@@ -2650,6 +2737,67 @@ camel_ews_store_maybe_disconnect (CamelEwsStore *store,
 		camel_service_disconnect_sync (service, FALSE, NULL, NULL);
 }
 
+static void
+ews_store_unset_oof_settings_state (CamelSession *session,
+				    GCancellable *cancellable,
+				    gpointer user_data,
+				    GError **error)
+{
+
+	CamelEwsStore *ews_store = user_data;
+	EEwsConnection *connection;
+	EEwsOofSettings *oof_settings;
+	EEwsOofState state;
+	GError *local_error = NULL;
+
+	camel_operation_push_message (cancellable, _("Unsetting the \"Out of Office\" status"));
+
+	connection = camel_ews_store_ref_connection (ews_store);
+	oof_settings = e_ews_oof_settings_new_sync (connection, cancellable, &local_error);
+	g_object_unref (connection);
+	if (local_error != NULL) {
+		g_propagate_error (error, local_error);
+		camel_operation_pop_message (cancellable);
+		return;
+	}
+
+	state = e_ews_oof_settings_get_state (oof_settings);
+	if (state == E_EWS_OOF_STATE_DISABLED) {
+		g_object_unref (oof_settings);
+		camel_operation_pop_message (cancellable);
+		return;
+	}
+
+	e_ews_oof_settings_set_state (oof_settings, E_EWS_OOF_STATE_DISABLED);
+	e_ews_oof_settings_submit_sync (oof_settings, cancellable, error);
+	g_object_unref (oof_settings);
+	if (local_error != NULL)
+		g_propagate_error (error, local_error);
+
+	camel_operation_pop_message (cancellable);
+
+}
+
+void
+camel_ews_store_unset_oof_settings_state (CamelEwsStore *ews_store)
+{
+	CamelService *service;
+	CamelSession *session;
+
+	g_return_if_fail (CAMEL_IS_EWS_STORE (ews_store));
+
+	service = CAMEL_SERVICE (ews_store);
+	session = camel_service_ref_session (service);
+
+	camel_session_submit_job (
+		session,
+		ews_store_unset_oof_settings_state,
+		g_object_ref (ews_store),
+		g_object_unref);
+
+	g_object_unref (session);
+}
+
 static void
 ews_store_dispose (GObject *object)
 {
@@ -2702,9 +2850,36 @@ camel_ews_store_class_init (CamelEwsStoreClass *class)
 	g_type_class_add_private (class, sizeof (CamelEwsStorePrivate));
 
 	object_class = G_OBJECT_CLASS (class);
+	object_class->set_property = ews_store_set_property;
+	object_class->get_property = ews_store_get_property;
 	object_class->dispose = ews_store_dispose;
 	object_class->finalize = ews_store_finalize;
 
+	g_object_class_install_property (
+		object_class,
+		PROP_HAS_OOO_SET,
+		g_param_spec_boolean (
+			"has-ooo-set",
+			"Has OOO Set",
+			"Has Out of Office state set",
+			FALSE,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT |
+			G_PARAM_STATIC_STRINGS));
+
+	g_object_class_install_property (
+		object_class,
+		PROP_OOO_ALERT_STATE,
+		g_param_spec_enum (
+			"ooo-alert-state",
+			"Out of Office Alert State",
+			"The state of the Out of Office Alert",
+			CAMEL_TYPE_EWS_STORE_OOO_ALERT_STATE,
+			CAMEL_EWS_STORE_OOO_ALERT_STATE_UNKNOWN,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT |
+			G_PARAM_STATIC_STRINGS));
+
 	service_class = CAMEL_SERVICE_CLASS (class);
 	service_class->settings_type = CAMEL_TYPE_EWS_SETTINGS;
 	service_class->query_auth_types_sync = ews_store_query_auth_types_sync;
