@@ -36,6 +36,7 @@
 G_DEFINE_TYPE (EEwsFolder, e_ews_folder, G_TYPE_OBJECT)
 
 struct _EEwsFolderPrivate {
+	GError *error;
 	gchar *name;
 	EwsFolderId *fid;
 	EwsFolderId *parent_fid;
@@ -69,6 +70,8 @@ e_ews_folder_finalize (GObject *object)
 
 	priv = folder->priv;
 
+	g_clear_error (&priv->error);
+
 	if (priv->name) {
 		g_free (priv->name);
 		priv->name = NULL;
@@ -115,7 +118,8 @@ e_ews_folder_init (EEwsFolder *folder)
 	priv = g_new0 (EEwsFolderPrivate, 1);
 	folder->priv = priv;
 
-	priv->folder_type = E_EWS_FOLDER_TYPE_MAILBOX;
+	priv->error = NULL;
+	priv->folder_type = E_EWS_FOLDER_TYPE_UNKNOWN;
 	priv->foreign = FALSE;
 }
 
@@ -158,7 +162,11 @@ e_ews_folder_set_from_soap_parameter (EEwsFolder *folder,
 		if (subparam) {
 			gchar *folder_class = e_soap_parameter_get_string_value (subparam);
 
-			if (g_strcmp0 (folder_class, "IPF.Contact") == 0) {
+			priv->folder_type = E_EWS_FOLDER_TYPE_UNKNOWN;
+
+			if (g_strcmp0 (folder_class, "IPF.Note") == 0 || (folder_class && g_str_has_prefix (folder_class, "IPF.Note."))) {
+				priv->folder_type = E_EWS_FOLDER_TYPE_MAILBOX;
+			} else if (g_strcmp0 (folder_class, "IPF.Contact") == 0) {
 				priv->folder_type = E_EWS_FOLDER_TYPE_CONTACTS;
 			} else if (g_strcmp0 (folder_class, "IPF.Appointment") == 0) {
 				priv->folder_type = E_EWS_FOLDER_TYPE_CALENDAR;
@@ -269,6 +277,35 @@ e_ews_folder_new_from_soap_parameter (ESoapParameter *param)
 	return folder;
 }
 
+EEwsFolder *
+e_ews_folder_new_from_error (const GError *error)
+{
+	EEwsFolder *folder;
+
+	g_return_val_if_fail (error != NULL, NULL);
+
+	folder = g_object_new (E_TYPE_EWS_FOLDER, NULL);
+	folder->priv->error = g_error_copy (error);
+
+	return folder;
+}
+
+gboolean
+e_ews_folder_is_error (EEwsFolder *folder)
+{
+	g_return_val_if_fail (E_IS_EWS_FOLDER (folder), TRUE);
+
+	return folder->priv->error != NULL;
+}
+
+const GError *
+e_ews_folder_get_error (EEwsFolder *folder)
+{
+	g_return_val_if_fail (E_IS_EWS_FOLDER (folder), NULL);
+
+	return folder->priv->error;
+}
+
 EwsFolderId *
 e_ews_folder_id_new (const gchar *id,
                      const gchar *change_key,
